package test.my_program;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import junit.framework.TestCase;
import ru.samsung.itschool.testbed.AndroidOutputStream;
import ru.samsung.itschool.testbed.TextFlusher;

/**
 * The Class AndroidOutputStreamTest.
 */
public class AndroidOutputStreamTest extends TestCase {

	/**
	 * The Class MockTextFlusher.
	 */
	private class MockTextFlusher implements TextFlusher {

		/** The flushed strings. */
		private List<String> flushedStrings = new ArrayList<String>();

		/* (non-Javadoc)
		 * @see ru.samsung.itschool.testbed.TextFlusher#flush(java.lang.String)
		 */
		@Override
		public void flush(String text) {
			flushedStrings.add(text);
		}

		/**
		 * Gets the flushed strings.
		 *
		 * @return the flushed strings
		 */
		public List<String> getFlushedStrings() {
			return Collections.unmodifiableList(flushedStrings);
		}

	}

	/** The Constant LINE_SEPARATOR. */
	private static final String LINE_SEPARATOR = System
			.getProperty("line.separator");

	/** The flusher. */
	private MockTextFlusher flusher;
	
	/** The output stream. */
	private OutputStream outputStream;
	
	/** The writer. */
	private PrintWriter writer;

	/* (non-Javadoc)
	 * @see junit.framework.TestCase#setUp()
	 */
	public void setUp() {
		this.flusher = new MockTextFlusher();
		this.outputStream = new AndroidOutputStream(flusher);
		this.writer = new PrintWriter(outputStream, true);
	}

	/* (non-Javadoc)
	 * @see junit.framework.TestCase#tearDown()
	 */
	public void tearDown() {
		this.writer.close();
	}

	/**
	 * Test simple flush.
	 */
	public void testSimpleFlush() {
		try {
			this.outputStream.write('a');
			this.outputStream.flush();

			String result = flusher.getFlushedStrings().get(0);
			String expected = "a";
			assertEquals(expected, result);
		} catch (IOException e) {
			fail(e.getMessage());
		}
	}

	/**
	 * Test cr flush.
	 */
	public void testCRFlush() {
		writer.println("abc");
		// must auto flush
		assertEquals("abc" + LINE_SEPARATOR, flusher
				.getFlushedStrings().get(0));
	}

	/**
	 * Test max size flush.
	 */
	public void testMaxSizeFlush() {
		String test = "0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789_";

		writer.print(test);
		writer.println();

		assertEquals(2, flusher.getFlushedStrings().size());
		assertEquals(test.substring(0, 100), flusher.getFlushedStrings().get(0));
		assertEquals("_" + LINE_SEPARATOR, flusher
				.getFlushedStrings().get(1));
	}
}
